## Citation: This lecture is heavily based off the tutorial from shiny.rstudio.com
## and can be found in Lesson 6 of the tutorial.

library(quantmod)
source("helpers.R")

## This file heavily relies on getSymbols which comes from quantmod
## and heavily relies on helpers.R to adjust prices for inflation.
## Imagine that we are using something similar to a streaming API

shinyServer(function(input, output) {
  ## Without the reactive calls, everytime any input value changes,
  ## Shiny will recall every segment of code. We have an API call
  ## out to yahoo from getSymbols, so we want to restrict this when
  ## possible. With the reactive calls:
  
  ## 1. A reactive expression saves its result the first time you run it.
  ## 2. The next time the reactive expression is called, it checks if the
  ##    saved value has become out of date (i.e., whether the widgets it
  ##    depends on have changed).
  ## 3. If the value is out of date, the reactive object will recalculate it
  ##    (and then save the new result).
  ## 4. If the value is up-to-date, the reactive expression will return the saved
  ##    value without doing any computation.
    
  ## When you click “Plot y axis on the log scale”, input$log will change and
  ## renderPlot will re-execute. Now
  ## 1. renderPlot will call dataInput()
  ## 2. dataInput will check that the dates and symb widgets have not changed
  ## 3. dataInput will return its saved data set of stock prices without
  ##    re-fetching data from Yahoo
  ## 4. renderPlot will re-draw the chart with the correct axis.  
  
  dataInput <- reactive({  
    getSymbols(input$symb, src = "yahoo", 
               from = input$dates[1],
               to = input$dates[2],
               auto.assign = FALSE)
  })
  
  finalInput <- reactive({
    if (!input$adjust) return(dataInput())
    adjust(dataInput())
  })
  
  output$plot <- renderPlot({
    chartSeries(finalInput(), name=input$symb, theme = chartTheme("white"), 
                type = "line", log.scale = input$log, TA = NULL)
  })
})